package APR::FunctionTable;

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# ! WARNING: this file was manually generated on
# !          Tue Jun 22 22:00:00 2004
# !          It contains a subset of functions appearing in
# !          ModPerl::FunctionTable used to build APR.so
# !          Eventually this will be autogenerated by
# !          Apache::ParseSource
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

$APR::FunctionTable = [
  {
    'return_type' => 'void',
    'name' => 'modperl_trace',
    'args' => [
      {
        'type' => 'const char *',
        'name' => 'func'
      },
      {
        'type' => 'const char *',
        'name' => 'fmt'
      },
      {
        'type' => '...',
        'name' => 'arg2'
      }
    ]
  },
  {
    'return_type' => 'void',
    'name' => 'modperl_trace_level_set',
    'args' => [
      {
        'type' => 'apr_file_t *',
        'name' => 'logfile'
      },
      {
        'type' => 'const char *',
        'name' => 'level'
      }
    ]
  },
  {
    'return_type' => 'void',
    'name' => 'modperl_trace_logfile_set',
    'args' => [
      {
        'type' => 'apr_file_t *',
        'name' => 'logfile_new'
      }
    ]
  },
  {
    'return_type' => 'unsigned long',
    'name' => 'modperl_debug_level',
    'args' => []
  },
  {
    'return_type' => 'SV *',
    'name' => 'modperl_hash_tie',
    'attr' => [
      '__inline__'
    ],
    'args' => [
      {
        'type' => 'PerlInterpreter *',
        'name' => 'my_perl'
      },
      {
        'type' => 'const char *',
        'name' => 'classname'
      },
      {
        'type' => 'SV *',
        'name' => 'tsv'
      },
      {
        'type' => 'void *',
        'name' => 'p'
      }
    ]
  },
  {
    'return_type' => 'void *',
    'name' => 'modperl_hash_tied_object',
    'attr' => [
      '__inline__'
    ],
    'args' => [
      {
        'type' => 'PerlInterpreter *',
        'name' => 'my_perl'
      },
      {
        'type' => 'const char *',
        'name' => 'classname'
      },
      {
        'type' => 'SV *',
        'name' => 'tsv'
      }
    ]
  },
  {
    'return_type' => 'SV *',
    'name' => 'modperl_perl_sv_setref_uv',
    'attr' => [
      '__inline__'
    ],
    'args' => [
      {
        'type' => 'PerlInterpreter *',
        'name' => 'my_perl'
      },
      {
        'type' => 'SV *',
        'name' => 'rv'
      },
      {
        'type' => 'const char *',
        'name' => 'classname'
      },
      {
        'type' => 'UV',
        'name' => 'uv'
      }
    ]
  },
  {
    'return_type' => 'modperl_uri_t *',
    'name' => 'modperl_uri_new',
    'attr' => [
      '__inline__'
    ],
    'args' => [
      {
        'type' => 'apr_pool_t *',
        'name' => 'p'
      }
    ]
  },
  {
    'return_type' => 'SV *',
    'name' => 'modperl_perl_gensym',
    'args' => [
      {
        'type' => 'PerlInterpreter *',
        'name' => 'my_perl'
      },
      {
        'type' => 'char *',
        'name' => 'pack'
      }
    ]
  },
  {
    'return_type' => 'char *',
    'name' => 'modperl_error_strerror',
    'args' => [
      {
        'type' => 'PerlInterpreter *',
        'name' => 'my_perl'
      },
      {
        'type' => 'apr_status_t',
        'name' => 'rc'
      }
    ]
  },
  {
    'return_type' => 'void',
    'name' => 'modperl_croak',
    'args' => [
      {
        'type' => 'PerlInterpreter *',
        'name' => 'my_perl'
      },
      {
        'type' => 'apr_status_t',
        'name' => 'rc'
      },
      {
        'type' => 'const char*',
        'name' => 'func'
      }
    ]
  },
  {
    'return_type' => 'apr_status_t',
    'name' => 'modperl_interp_unselect',
    'args' => [
      {
        'type' => 'void *',
        'name' => 'data'
      }
    ]
  },
  {
    'return_type' => 'apr_bucket *',
    'name' => 'modperl_bucket_sv_create',
    'args' => [
      {
        'type' => 'PerlInterpreter *',
        'name' => 'my_perl'
      },
      {
        'type' => 'apr_bucket_alloc_t *',
        'name' => 'list'
      },
      {
        'type' => 'SV *',
        'name' => 'sv'
      },
      {
        'type' => 'apr_off_t',
        'name' => 'offset'
      },
      {
        'type' => 'apr_size_t',
        'name' => 'len'
      }
    ]
  },
];

1;
