# 
# /*
#  * *********** WARNING **************
#  * This file generated by ModPerl::WrapXS/0.01
#  * Any changes made here will be lost
#  * ***********************************
#  * 01: lib/ModPerl/Code.pm:709
#  * 02: /Users/phred/dev/svn/modperl/mod_perl-2.0/blib/lib/ModPerl/WrapXS.pm:1135
#  * 03: /Users/phred/dev/svn/modperl/mod_perl-2.0/blib/lib/ModPerl/WrapXS.pm:1186
#  * 04: Makefile.PL:427
#  * 05: Makefile.PL:329
#  * 06: Makefile.PL:58
#  */
# 

package APR::Util;
$APR::Util::VERSION               = 0.009000;

package APR::Base64;
$APR::Base64::VERSION             = 0.009000;

package Apache2::Process;
$Apache2::Process::VERSION        = 2.000007;

package Apache2::HookRun;
$Apache2::HookRun::VERSION        = 2.000007;

package APR::Bucket;
$APR::Bucket::VERSION             = 0.009000;

package Apache2::Command;
$Apache2::Command::VERSION        = 2.000007;

package Apache2::ConnectionUtil;
$Apache2::ConnectionUtil::VERSION = 2.000007;

package APR::Finfo;
$APR::Finfo::VERSION              = 0.009000;

package APR::SockAddr;
$APR::SockAddr::VERSION           = 0.009000;

package ModPerl::Global;
$ModPerl::Global::VERSION         = 2.000007;

package Apache2::FilterRec;
$Apache2::FilterRec::VERSION      = 2.000007;

package APR::BucketType;
$APR::BucketType::VERSION         = 0.009000;

package Apache2::MPM;
$Apache2::MPM::VERSION            = 2.000007;

package Apache2::Directive;
$Apache2::Directive::VERSION      = 2.000007;

package APR::Error;
$APR::Error::VERSION              = 0.009000;

package Apache2::Filter;
$Apache2::Filter::VERSION         = 2.000007;

package Apache2::Util;
$Apache2::Util::VERSION           = 2.000007;

package APR::Date;
$APR::Date::VERSION               = 0.009000;

package Apache2::CmdParms;
$Apache2::CmdParms::VERSION       = 2.000007;

package Apache2::Log;
$Apache2::Log::VERSION            = 2.000007;

package Apache2::ServerUtil;
$Apache2::ServerUtil::VERSION     = 2.000007;

package Apache2::RequestUtil;
$Apache2::RequestUtil::VERSION    = 2.000007;

package Apache2::URI;
$Apache2::URI::VERSION            = 2.000007;

package APR::UUID;
$APR::UUID::VERSION               = 0.009000;

package APR::Table;
$APR::Table::VERSION              = 0.009000;

package APR::Status;
$APR::Status::VERSION             = 0.009000;

package Apache2::Connection;
$Apache2::Connection::VERSION     = 2.000007;

package Apache2::ServerRec;
$Apache2::ServerRec::VERSION      = 2.000007;

package ModPerl::Util;
$ModPerl::Util::VERSION           = 2.000007;

package Apache2::SubRequest;
$Apache2::SubRequest::VERSION     = 2.000007;

package APR::URI;
$APR::URI::VERSION                = 0.009000;

package Apache2::RequestIO;
$Apache2::RequestIO::VERSION      = 2.000007;

package APR::Brigade;
$APR::Brigade::VERSION            = 0.009000;

package APR::ThreadMutex;
$APR::ThreadMutex::VERSION        = 0.009000;

package APR::BucketAlloc;
$APR::BucketAlloc::VERSION        = 0.009000;

package Apache2::SubProcess;
$Apache2::SubProcess::VERSION     = 2.000007;

package Apache2::Response;
$Apache2::Response::VERSION       = 2.000007;

package APR::IpSubnet;
$APR::IpSubnet::VERSION           = 0.009000;

package APR::ThreadRWLock;
$APR::ThreadRWLock::VERSION       = 0.009000;

package Apache2::RequestRec;
$Apache2::RequestRec::VERSION     = 2.000007;

package Apache2::Module;
$Apache2::Module::VERSION         = 2.000007;

package APR::OS;
$APR::OS::VERSION                 = 0.009000;

package APR::Socket;
$APR::Socket::VERSION             = 0.009000;

package Apache2::Access;
$Apache2::Access::VERSION         = 2.000007;

package APR::String;
$APR::String::VERSION             = 0.009000;

package APR::Pool;
$APR::Pool::VERSION               = 0.009000;

package ModPerl::MethodLookup;
$ModPerl::MethodLookup::VERSION   = 2.000007;

