package TestAPRlib::os;

# testing APR::OS API

use strict;
use warnings FATAL => 'all';

use Apache::Test;
use Apache::TestUtil;

use APR::OS ();

sub num_of_tests {
    return 1;
}

sub test {
    ok 1;
}

1;
